@@ -1,7 +1,7 @@
 /*  
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc-Oliver Ihm 1995-2002
+    written by Marc-Oliver Ihm 1995-2004
     homepage: www.yabasic.de
     
     function.c --- code for functions
@@ -124,8 +124,6 @@ void token(struct command *cmd) /* extract token from variable */
       }
       if (!*p) break;
     }
-  } else {
-    pp[1][0]='\0';
   }
       
   ar->bounds[0]=num+1;
@@ -135,6 +133,7 @@ void token(struct command *cmd) /* extract token from variable */
   s->value=num;
 }
 
+
 void tokenalt(struct command *cmd) /* extract token from variable with alternate semantics */
 {
   char *del; /* delimiter for strings */
@@ -330,7 +329,8 @@ void function(struct command *current) /* performs a function */
   time_t datetime;
   int type,result,len,start,i,max;
   char *str,*str2;
-  
+
+  a3=NULL;
   type=current->args;
   if (type>fTHREEARGS) a4=pop(stSTRING_OR_NUMBER);
   if (type>fTWOARGS) a3=pop(stSTRING_OR_NUMBER);
@@ -370,6 +370,10 @@ void function(struct command *current) /* performs a function */
     value=log(a1->value);
     result=stNUMBER;
     break;
+  case fLOG2:
+    value=log(a1->value)/log(a2->value);
+    result=stNUMBER;
+    break;
   case fLEN:
     value=(double) strlen(a1->pointer);
     result=stNUMBER;
@@ -380,8 +384,9 @@ void function(struct command *current) /* performs a function */
     result=stSTRING;
     break;
   case fSTR2:
+  case fSTR3:
     result=stSTRING;
-    if (!myformat(string,a1->value,a2->pointer)) {
+    if (!myformat(string,a1->value,a2->pointer,a3?a3->pointer:NULL)) {
       pointer=my_strdup("");
       sprintf(string,"'%s' is not a valid format",(char *)a2->pointer);
       error(ERROR,string);
@@ -394,7 +399,7 @@ void function(struct command *current) /* performs a function */
     result=stNUMBER;
     break;
   case fSQR:
-    value=exp(log(a1->value)/2);
+    value=a1->value*a1->value;
     result=stNUMBER;
     break;
   case fINT:
@@ -430,7 +435,7 @@ void function(struct command *current) /* performs a function */
     result=stNUMBER;
     break;
   case fRAN2:
-    value=(float)rand()/RAND_MAX;
+    value=(double)rand()/RAND_MAX;
     result=stNUMBER;
     break;
   case fMIN:
@@ -737,6 +742,7 @@ static int do_system2(char *cmd) /* execute command as system */
   DWORD ec; /* exit code */
   SECURITY_ATTRIBUTES prosec;
   SECURITY_ATTRIBUTES thrsec;
+  char *comspec;
 		
   ZeroMemory(&prosec,sizeof(prosec));
   prosec.nLength=sizeof(prosec);
@@ -750,7 +756,9 @@ static int do_system2(char *cmd) /* execute command as system */
   start.hStdOutput=GetStdHandle(STD_OUTPUT_HANDLE);
   start.hStdError=GetStdHandle(STD_ERROR_HANDLE);
   start.hStdInput=GetStdHandle(STD_INPUT_HANDLE);
-  sprintf(string,"command.com /C %s",cmd);
+  comspec=getenv("COMSPEC");
+  if (!comspec) comspec="command.com";
+  sprintf(string,"%s /C %s",comspec,cmd);
   if (!CreateProcess(NULL,string,&prosec,&thrsec,TRUE,0,
     NULL,NULL,&start,&proc)) {
     sprintf(string,"couldn't execute '%s'",cmd);
@@ -758,10 +766,10 @@ static int do_system2(char *cmd) /* execute command as system */
     return -1;
   }
   WaitForSingleObject(proc.hProcess,INFINITE);
+  if (!GetExitCodeProcess(proc.hProcess,&ec)) ec=-1;
   CloseHandle(proc.hProcess);
   CloseHandle(proc.hThread);
-  if (GetExitCodeProcess(proc.hProcess,&ec)) return ec;
-  return -1;
+  return ec;
 #endif  
 }
 
@@ -817,6 +825,7 @@ static char *do_system(char *cmd) /* executes command via command.com */
   HANDLE piperead,pipewrite; /* both ends of pipes */
   SECURITY_ATTRIBUTES prosec;
   SECURITY_ATTRIBUTES thrsec;
+  char *comspec;
 #endif
   
   clear_buff();
@@ -861,7 +870,9 @@ static char *do_system(char *cmd) /* executes command via command.com */
   start.hStdError=pipewrite;
   start.hStdInput=GetStdHandle(STD_INPUT_HANDLE);
 		
-  sprintf(string,"command.com /C %s",cmd);
+  comspec=getenv("COMSPEC");
+  if (!comspec) comspec="command.com";
+  sprintf(string,"%s /C %s",comspec,cmd);
   if (!CreateProcess(NULL,string,&prosec,&thrsec,TRUE,0,
     NULL,NULL,&start,&proc)) {
     sprintf(string,"couldn't execute '%s'",cmd);
@@ -957,7 +968,7 @@ static double other2dec(char *hex,int base) /* convert hex or binary to double n
     dec*=base;
     found=strchr(digits,tolower(hex[i]));
     if (!found || found-digits>=base) {
-      sprintf(string,"Not a hex number: '%s'",hex);
+      sprintf(string,"Not a base-%d number: '%s'",base,hex);
       error(ERROR,string);
       return 0.;
     }
@@ -967,15 +978,15 @@ static double other2dec(char *hex,int base) /* convert hex or binary to double n
 }
 
 
-int myformat(char *dest,double num,char *format) /* format number according to string */
+int myformat(char *dest,double num,char *format,char *sep) /* format number according to string */
 {
   int i1,i2; /* dummy */
   char c1; /* dummy */
   static char ctrl[6];
   char *found,*form;
-  int pre,post,dot,len,i,digit;
-  int ip,neg=FALSE;
-  double fp,round;
+  int pre,post,dot,len,i,j,digit,colons,dots;
+  int neg=FALSE;
+  double ip,fp,round;
   static char *digits="0123456789";
   
   form=format;
@@ -987,9 +998,9 @@ int myformat(char *dest,double num,char *format) /* format number according to s
       form++;
     } 
     if (sscanf(form,"%u.%u%c%n",&i1,&i2,&c1,&i)!=3 &&
-      sscanf(form,"%u.%c%n",&i2,&c1,&i)!=2 &&
-      sscanf(form,".%u%c%n",&i2,&c1,&i)!=2 &&
-      sscanf(form,"%u%c%n",&i2,&c1,&i)!=2) return FALSE;
+	sscanf(form,"%u.%c%n",&i2,&c1,&i)!=2 &&
+	sscanf(form,".%u%c%n",&i2,&c1,&i)!=2 &&
+	sscanf(form,"%u%c%n",&i2,&c1,&i)!=2) return FALSE;
     if (!strchr("feEgG",c1) || form[i]) return FALSE;
     /* seems okay, let's print */
     sprintf(dest,format,num);
@@ -998,49 +1009,79 @@ int myformat(char *dest,double num,char *format) /* format number according to s
       neg=TRUE;
       num=-num;
     }
-    for(pre=0;*format=='#';pre++) format++;
-    dot=*format;
-    if (*format) {
-      if (*format!='.') return FALSE;
-      format++;
+    colons=0;
+    dots=0;
+    pre=0;
+    post=0;
+    for(form=format;*form;form++) {
+      if (*form==',') {
+	if (dots) return FALSE;
+	colons++;
+      } else if (*form=='.') {
+	dots++;
+      } else if (*form=='#') {
+	if (dots) 
+	  post++;
+	else
+	  pre++;
+      } else {
+	return FALSE;
+      }
     }
-    for(post=0;*format=='#';post++) format++;
-    if (dot) *(dest+pre)=dot;
-    len=pre+post+(dot?1:0);
+    if (dots>1) return FALSE;
+    len=strlen(format);
     dest[len]='\0';
     round=0.5;
     for(i=0;i<post;i++) round/=10.;
     if (fabs(num)<round) neg=FALSE;
     num+=round;
-    ip=(int)num;
+    ip=floor(num);
     fp=num-ip;
-    if (ip) {
-      for(i=pre-1;i>=0;i--) {
-	if (ip) {
-	  digit=ip%10;
-	  ip/=10;
-	  dest[i]=digits[digit];
+    if (fp>1 || fp<0) fp=0;
+    dest[pre+colons]=format[pre+colons];
+    if ((int)ip) {
+      for(i=pre+colons-1;i>=0;i--) {
+	if (neg && !(int)ip) {
+	  neg=0;
+	  dest[i]='-';
 	} else {
-	  break;
+	  if (format[i]=='#') {
+	    digit=((int)ip)%10;
+	    ip/=10;
+	    if (((int)ip) || digit>0) 
+	      dest[i]=digits[digit];
+	    else
+	      dest[i]=' ';
+	  } else {
+	    if ((int)ip)
+	      dest[i]=format[i];
+	    else
+	      dest[i]=' ';
+	  }
 	}
       }
     } else {
-      i=pre-1;
+      i=pre+colons-1;
       dest[i--]='0';
     }
-    if ((neg && i<0) || ip) {
-      for(i=0;i<len;i++) dest[i]='*';
+    if ((neg && i<0) || ((int)ip)) {
+      strcpy(dest,format);
       return TRUE;
     }
     if (neg) dest[i--]='-';
     for(;i>=0;i--) dest[i]=' ';
-    for(i=pre+1;i<len;i++) {
+    for(i=pre+colons+1;i<len;i++) {
       fp*=10;
       digit=(int)fp;
       fp-=digit;
       dest[i]=digits[digit];
     }
-    return TRUE;
+    if (sep && sep[0] && sep[1]) {
+      for(i=0;i<len;i++) {
+	if (dest[i]==',') dest[i++]=sep[0];
+	if (dest[i]=='.') dest[i++]=sep[1];
+      }
+    }
   }
   return TRUE;
 }
@@ -1119,7 +1160,7 @@ void create_poke(char flag) /* create Command 'cPOKE' */
 }
 
 
-void poke(struct command *cmd) /* poke in internals */
+void poke(struct command *cmd) /* poke into internals */
 {
   char *dest,*s,c;
   char *sarg=NULL;
@@ -1249,12 +1290,13 @@ static double peek(char *dest) /* peek into internals */
   else if (!strcmp(dest,"fontheight")) return fontheight;
   else if (!strcmp(dest,"screenheight")) return LINES;
   else if (!strcmp(dest,"screenwidth")) return COLS;
-  else if (!strcmp(dest,"argument")) return yabargc;
+  else if (!strcmp(dest,"argument") || !strcmp(dest,"arguments") ) return yabargc;
   else if (!strcmp(dest,"version")) return strtod(VERSION,NULL);
   else if (!strcmp(dest,"error")) return errorcode;
   else if (!strcmp(dest,"read_controls")) return read_controls;
+  else if (!strcmp(dest,"isbound")) return is_bound; 
   else if (dest[0]=='#') {
-    error(ERROR,"don't use quotes when peeking within file");
+    error(ERROR,"don't use quotes when peeking into a file");
     return 0;
   }
   
@@ -1300,7 +1342,7 @@ static char *peek2(char *dest,struct command *curr) /* peek into internals */
 #endif
   }
   else if (!strcmp(dest,"font")) return my_strdup(font);
-  else if (!strcmp(dest,"argument")) {
+  else if (!strcmp(dest,"argument") || !strcmp(dest,"arguments")) {
     if (yabargc>0) {
       s=yabargv[0];
       yabargc--;
@@ -1636,10 +1678,3 @@ void boole(struct command *type)  /* perform and/or/not */
 }    
 
 
-
-
-
-
-
-
-
