@@ -1,10 +1,10 @@
 /*  
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc-Oliver Ihm 1995-2002
+    written by Marc-Oliver Ihm 1995-2004
     homepage: www.yabasic.de
     
-    grafic.c --- code for windowed graphics, printing and plotting
+    graphic.c --- code for windowed graphics, printing and plotting
     
     This file is part of yabasic and may be copied only 
     under the terms of either the Artistic License or 
@@ -28,6 +28,9 @@
 #endif
 #ifdef UNIX
 #include <curses.h>
+#include <sys/types.h>
+#include <sys/stat.h>
+#include <unistd.h>
 #ifndef KEY_MAX
 #define KEY_MAX 0777
 #endif
@@ -38,7 +41,7 @@
 
 static void transform(double *,double *); /* do transformation */
 static void itransform(int *,int *); /* integer variant of transform() */
-static int grafinit(void);  /* initialize grafics (either X or WIN95) */
+static int grafinit(void);  /* initialize grafics (either X or Windows) */
 static void addbits(char *,int); /* add one bit to bitstring */
 static char *newbits(int,int); /* create a new bitmap string */
 static int readbits(char *); /* read bits from string one after another */
@@ -51,6 +54,7 @@ static void startdraw(int); /* prepare for drawing */
 
 /* printing and plotting */
 static char *printerfilename=NULL; /* filename to print on */
+static int deleteprinterfile=FALSE; /* true, if we created the filename ourself an can delete it on close */
 int print_to_file=FALSE; /* print to file ? */
 #ifdef WINDOWS
 HFONT printerfont; /* handle of printer-font */
@@ -527,7 +531,7 @@ char *getreg(char *name) /* get defaults from Registry */
 #endif
 
 
-static int grafinit(void)  /* initialize grafics (either X or WIN95) */
+static int grafinit(void)  /* initialize grafics (either X or Windows) */
 {
   
 #ifdef UNIX
@@ -758,6 +762,8 @@ void line(struct command *cmd) /* draw a line */
   double x1,y1,x2,y2;
   static double lastx,lasty;
   static int lastvalid=FALSE;
+  static double initialx,initialy;
+  static int initialvalid=FALSE;  
   int numpoints;
   int clear;
   
@@ -768,28 +774,42 @@ void line(struct command *cmd) /* draw a line */
   
   clear=cmd->tag & dmCLEAR;
   numpoints=cmd->args;
-  if (numpoints==0) {
+  if (numpoints==-1) { /* new curve */
     lastvalid=FALSE;
+    initialvalid=FALSE;
     return;
-  }
-  y2=pop(stNUMBER)->value;
-  x2=pop(stNUMBER)->value;
-  if (numpoints==1) {
-    if (!lastvalid) {
-      lastx=x2;
-      lasty=y2;
-      lastvalid=TRUE;
-      return;
-    }
-    y1=lasty;
+  } else if (numpoints==0) { /* close curve */
+    if (!lastvalid || !initialvalid) return;
+    lastvalid=FALSE;
+    initialvalid=FALSE;
     x1=lastx;
-  }
-  else {
-    y1=pop(stNUMBER)->value;
-    x1=pop(stNUMBER)->value;
+    y1=lasty;
+    x2=initialx;
+    y2=initialy;
+  } else { /* line to x2,y2 */
+    y2=pop(stNUMBER)->value;
+    x2=pop(stNUMBER)->value;
+    if (numpoints==1) {
+      if (!lastvalid) {
+	initialx=lastx=x2;
+	initialy=lasty=y2;
+	initialvalid=lastvalid=TRUE;
+	return;
+      }
+      y1=lasty;
+      x1=lastx;
+    } else { /* line x1,y1 to x2,y2 */
+      y1=pop(stNUMBER)->value;
+      x1=pop(stNUMBER)->value;
+    }
   }
   lastx=x2;
   lasty=y2;
+  if (!initialvalid) {
+    initialx=x1;
+    initialy=y2;
+    initialvalid=TRUE;
+  }
   transform(&x1,&y1);
   transform(&x2,&y2);
   
@@ -1031,10 +1051,10 @@ void closewin() /* close the window */
   XFreePixmap(display,backbit);
   XDestroyWindow(display,window); 
   XFlush(display);
+  if (printerfile) closeprinter();
 #else /* WINDOWS */
   PostThreadMessage(wtid,WM_QUIT,0,0);
 #endif
-  closeprinter();
 }
 
 
@@ -1513,12 +1533,32 @@ void openprinter(struct command *cmd) /* opens a printer */
 #ifdef UNIX
 
   if (*printerfilename=='\0') {
+#ifdef HAVE_TMPNAM
+    char *pfn;
+    pfn=tmpnam(NULL);
+    printerfilename=my_strdup(pfn);
+#else
+    struct stat s;
     my_free(printerfilename);
     printerfilename=my_strdup("/tmp/yabasic.ps");
+    if (stat(printerfilename,&s) && errno!=ENOENT) {
+      sprintf(string,"could not check printerfile '%s': %s",printerfilename,my_strerror(errno));
+      error(ERROR,string);
+      return;
+    }
+    if (s.st_mode & S_IFLNK) {
+      sprintf(string,"could not print to file '%s'; it is a symbolic link");
+      error(ERROR,string);
+      return;
+    }
+#endif
+    deleteprinterfile=TRUE;
+  } else {
+    deleteprinterfile=FALSE;
   }
   printerfile=fopen(printerfilename,"w");
   if (!printerfile) {
-    sprintf(string,"could not open file '%s' for printing",printerfilename);
+    sprintf(string,"could not open file '%s' for printing: %s",printerfilename,my_strerror(errno));
     error(ERROR,string);
   }
 
@@ -1671,7 +1711,8 @@ void closeprinter() /* close printer */
     fprintf(printerfile,"showpage\ngrestore\n%%%%Trailer\n");
     fclose(printerfile);
     printerfile=NULL;
-    if (!strncmp(printerfilename,"/tmp/",5)) {
+    if (deleteprinterfile) {
+      deleteprinterfile=FALSE;
       sprintf(string,"lpr %s",printerfilename);
       if (system(string)) {
         sprintf(string,"couldn't print '%s'",printerfilename);
