@@ -1,7 +1,7 @@
 /*  
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc-Oliver Ihm 1995-2002
+    written by Marc-Oliver Ihm 1995-2004
     homepage: www.yabasic.de
     
     io.c --- code for screen and file i/o
@@ -127,7 +127,7 @@ void create_print(char type) /* create command 'print' */
 void print(struct command *cmd) /* print on screen */
 {
   int type;
-  struct stackentry *p,*q;
+  struct stackentry *p,*q,*r;
   static int last='n';
   char *s;
   int x,y;
@@ -140,6 +140,7 @@ void print(struct command *cmd) /* print on screen */
   CHAR_INFO ci;
 #endif
   
+  r=NULL;
   type=cmd->tag;
   if (!checkstream()) return;
   switch(type) {
@@ -203,6 +204,8 @@ void print(struct command *cmd) /* print on screen */
       sprintf(string,"%s%g",(last=='d')?" ":"",d);
     onestring(string);
     break;
+  case 'U':
+    r=pop(stSTRING);    
   case 'u':  /* print using */
     p=pop(stSTRING);
     q=pop(stNUMBER);
@@ -212,7 +215,7 @@ void print(struct command *cmd) /* print on screen */
       *s=' ';
       s++;
     }
-    if (!myformat(s,q->value,p->pointer)) {
+    if (!myformat(s,q->value,p->pointer,r?r->pointer:NULL)) {
       sprintf(string,"'%s' is not a valid format",(char *)p->pointer);
       error(ERROR,string);
       break;
@@ -262,7 +265,7 @@ void clearscreen() /* clear entire screen */
   DWORD written; /* number of chars actually written */
   COORD coord; /* coordinates to start writing */
 #endif
-  
+
   if (!curinized) curinit();
 #ifdef UNIX
   clear();
@@ -270,11 +273,11 @@ void clearscreen() /* clear entire screen */
 #else
   coord.X=0;
   coord.Y=0;
-  FillConsoleOutputCharacter(ConsoleOutput,' ',LINES*COLS,
-    coord,&written);
+  FillConsoleOutputCharacter(ConsoleOutput,' ',LINES*COLS,coord,&written);
   FillConsoleOutputAttribute(ConsoleOutput,
-    FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE,LINES*COLS,
-    coord,&written);
+			     FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_BLUE,
+			     LINES*COLS,
+			     coord,&written);
   SetConsoleCursorPosition(ConsoleOutput,coord);
 #endif
 }
@@ -707,7 +710,7 @@ void myopen(struct command *cmd) /* open specified file for given name */
   } else {
     handle=fopen(name,mode);
     if (handle==NULL) {
-      sprintf(errorstring,"could not open '%s'",name);
+      sprintf(errorstring,"could not open '%s': %s",name,my_strerror(errno));
       errorcode=4;
       goto open_done;
     }
@@ -727,7 +730,7 @@ void checkopen(void) /* check, if open has been sucessfull */
   double result;
   
   result=pop(stNUMBER)->value;
-  if (result<0) {
+  if (result<=0) {
     error(ERROR,errorstring);
   } 
 }
@@ -932,7 +935,7 @@ void testeof(struct command *cmd) /* close the specified stream */
 int badstream(int stream,int errcode) /* test for valid stream id */
 {
   if (stream!=STDIO_STREAM && (stream>FOPEN_MAX-4 || stream<=0)) {
-    sprintf(errcode?errorstring:string,"invalid stream: %d (can handle only streams from 0 to %d)",stream,FOPEN_MAX-4);
+    sprintf(errcode?errorstring:string,"invalid stream: %d (can handle only streams from 1 to %d)",stream,FOPEN_MAX-4);
     if (errcode) 
       errorcode=errcode;
     else
@@ -1206,14 +1209,14 @@ void colour(struct command *cmd) /* switch on colour */
     }
     fc=name2yc(fore);
     if (fc<0) {
-      sprintf(string,"unknown colour: '%s'",fore);
+      sprintf(string,"unknown foreground colour: '%s'",fore);
       error(ERROR,string);
     }
     bc=stdbc;
     if (back) {
       bc=name2yc(back);
       if (fc<0) {
-	sprintf(string,"unknown colour: '%s'",back);
+	sprintf(string,"unknown background colour: '%s'",back);
 	error(ERROR,string);
       }
     }
@@ -1248,6 +1251,8 @@ static void initcol(void) /* initialize curses colors */
       init_pair(i*8+j,yc2oc(i,TRUE),yc2oc(j,FALSE));
     }
   }
+  init_color(COLOR_YELLOW,1000,1000,0);
+
   col=inch()&A_COLOR;
   pair_content(col,&f,&b);
   stdfc=oc2yc(f);
@@ -1297,7 +1302,7 @@ int yc2oc(int yc,int fore) /* convert a yabasic color to operating system color
     if (yc==YC_RED) return FOREGROUND_RED;
     if (yc==YC_BLUE) return FOREGROUND_BLUE;
     if (yc==YC_GREEN) return FOREGROUND_GREEN;
-    if (yc==YC_YELLOW) return FOREGROUND_GREEN | FOREGROUND_RED;
+    if (yc==YC_YELLOW) return FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_INTENSITY;
     if (yc==YC_CYAN) return FOREGROUND_GREEN | FOREGROUND_BLUE;
     if (yc==YC_MAGENTA) return FOREGROUND_BLUE | FOREGROUND_RED;
   } else {
@@ -1306,7 +1311,7 @@ int yc2oc(int yc,int fore) /* convert a yabasic color to operating system color
     if (yc==YC_RED) return BACKGROUND_RED;
     if (yc==YC_BLUE) return BACKGROUND_BLUE;
     if (yc==YC_GREEN) return BACKGROUND_GREEN;
-    if (yc==YC_YELLOW) return BACKGROUND_GREEN | BACKGROUND_RED;
+    if (yc==YC_YELLOW) return BACKGROUND_GREEN | BACKGROUND_RED | BACKGROUND_INTENSITY;
     if (yc==YC_CYAN) return BACKGROUND_GREEN | BACKGROUND_BLUE;
     if (yc==YC_MAGENTA) return BACKGROUND_BLUE | BACKGROUND_RED;
   }
@@ -1337,7 +1342,9 @@ int oc2yc(int oc) /* convert an operating system color to yabasic color */
   if (oc==(FOREGROUND_GREEN) || 
       oc==(BACKGROUND_GREEN)) return YC_GREEN;
   if (oc==(FOREGROUND_RED|FOREGROUND_GREEN) || 
-      oc==(BACKGROUND_RED|BACKGROUND_GREEN)) return YC_YELLOW;
+      oc==(BACKGROUND_RED|BACKGROUND_GREEN) ||
+      oc==(FOREGROUND_RED|FOREGROUND_GREEN|FOREGROUND_INTENSITY) || 
+      oc==(BACKGROUND_RED|BACKGROUND_GREEN|BACKGROUND_INTENSITY)) return YC_YELLOW;
   if (oc==(FOREGROUND_BLUE|FOREGROUND_GREEN) || 
       oc==(BACKGROUND_BLUE|BACKGROUND_GREEN)) return YC_CYAN;
   if (oc==(FOREGROUND_RED|FOREGROUND_BLUE) || 
@@ -1414,7 +1421,7 @@ void putchars(void) /* put rect onto screen */
   }
 #ifdef UNIX
     if (has_colors()) 
-      attrset(A_NORMAL | COLOR_PAIR(stdfc*8+stdbc));
+      attrset(A_NORMAL|COLOR_PAIR(stdfc*8+stdbc));
     else
       attrset(A_NORMAL);
     move(y,x);
