@@ -1,15 +1,15 @@
 /*  
 
-    YABASIC ---  a simple Basic Interpreter
-    written by Marc-Oliver Ihm 1995-2002
-    homepage: www.yabasic.de
+YABASIC ---  a simple Basic Interpreter
+written by Marc-Oliver Ihm 1995-2004
+homepage: www.yabasic.de
     
-    main.c --- main() and auxilliary functions
+main.c --- main() and auxilliary functions
     
-    This file is part of yabasic and may be copied only 
-    under the terms of either the Artistic License or 
-    the GNU General Public License (GPL), both of which 
-    can be found at www.yabasic.de
+This file is part of yabasic and may be copied only 
+under the terms of either the Artistic License or 
+the GNU General Public License (GPL), both of which 
+can be found at www.yabasic.de
 
 */
 
@@ -23,16 +23,14 @@
 /* ------------- defines ---------------- */
 
 #define DONE {current=current->next;break;}  /* reduces type-work */
-#ifdef UNIX
-#define ARCHITECTURE UNIX_ARCHITECTURE 
-#else
-#define ARCHITECTURE WINDOWS_ARCHITECTURE 
-#endif
+#define COPYRIGHT "Copyright 1995-2004 by Marc-Oliver Ihm"
 #define BANNER \
 "\nThis is yabasic version " VERSION ",\nbuilt on "\
-ARCHITECTURE " at " BUILD_TIME "\n\n"\
-"      Copyright 1995-2002 by Marc-Oliver Ihm\n\n"
+ARCHITECTURE " at " BUILD_TIME "\n\n      " COPYRIGHT "\n\n"
+#define BANNER_VERSION \
+"yabasic " VERSION ", built on " ARCHITECTURE "\n" COPYRIGHT
 #define YABFORHELP "(type 'yabasic -help' for help)"
+#define YABMAGIC "__YaBaSiC_MaGiC_CoOkIe__"
 
 /* ------------- external references ---------------- */
 
@@ -46,14 +44,15 @@ static void std_diag(char *,int,char *); /* produce standard diagnostic */
 static void parse_arguments(int,char *argv[]); /* parse cmd line arguments */
 static void initialize(void); /* give correct values to pointers etc ... */
 static void run_it(void); /* execute the compiled code */
-static void end_it(int); /* perform shutdown operations */
+static void end_it(void); /* perform shutdown operations */
 #ifdef WINDOWS
 static void chop_command(char *,int *,char ***); /* chops WIN95-commandline */
 #endif
 void create_docu_array(void); /* create array with documentation */
 int equal(char *,char *,int); /* helper for processing options */
 void do_help(char *); /* process help option */ 
-
+static int mybind(char *); /* bind a program to the interpreter and save it */
+char *find_interpreter(char *); /* find interpreter with full path */
 
 /* ------------- global variables ---------------- */
 
@@ -68,12 +67,15 @@ static int note_count; /* number of notes */
 static int warning_count; /* number of warning messages */
 static int error_count; /* number of error messages */
 int interactive; /* true, if commands come from stdin */
+int is_bound; /* true, if this executable is bound */
+static char *to_bind=NULL; /* name bound program to be written */
+FILE *bound_program=NULL; /* points to embedded yabasic program (if any) */
 char *string; /* for trash-strings */
 char *errorstring; /* for error-strings */
 int errorcode; /* error-codes */
 static int commandcount; /* total number of commands */
 int program_state;  /* state of program */
-char *progname; /* name of yabasic-program */
+char *progname=NULL; /* name of yabasic-program */
 int print_docu=FALSE; /* TRUE, if only docu should be printed */
 int hold_docu=FALSE; /* TRUE, if docu should be printed in portions */
 #ifdef WINDOWS
@@ -93,6 +95,8 @@ char library_default[200]; /* default full path to search libraries */
 static struct command *docuhead=NULL; /* first docu in main */
 static int docucount=0; /* number of docu-lines in array */
 int check_compat=0; /* true, if compatibility should be checked */
+static char *interpreter_path=NULL; /* name of interpreter executing; i.e. ARGV[0] */
+static char *main_file_name=NULL; /* name of program to be executed */
 
 /* ------------- main program ---------------- */
 
@@ -164,6 +168,10 @@ int main(int argc,char **argv)
     fromlibpath=FALSE;
   }
 #endif
+  
+  /* find out, if this executable is bound to a yabasic program */
+  interpreter_path=find_interpreter(argv[0]);
+  is_bound=isbound();
 
   /* parse arguments */
   parse_arguments(argc,argv);
@@ -186,15 +194,20 @@ int main(int argc,char **argv)
   program_state=INITIALIZED;
 
   error(NOTE,"calling parser/compiler");
+      
   if (interactive) {
     printf("%s",BANNER);
-    printf("Enter your program and type RETURN twice when done.\n");
-    printf("Your program will be executed immediately and cannot be saved;\ntype 'yabasic -help' for more info on using yabasic.\n\n");
+    printf("Enter your program and type RETURN twice when done.\n\n");
+    printf("Your program will execute immediately and CANNOT BE SAVED;\n");
+    printf("create your program with an EXTERNAL EDITOR, if you want to keep it.\n");
+#ifdef UNIX
+    printf("Type 'man yabasic' or see the file yabasic.htm for more information.\n\n");
+#else
+    printf("See the documentation within the start-menu for more information.\n\n");
+#endif
   }
   program_state=COMPILING;
-  if (yyparse() && errorlevel>ERROR) {
-    error(ERROR,"Couldn't parse program");
-  }
+  if (yyparse() && errorlevel>ERROR) error(ERROR,"Couldn't parse program");
 
   if (errorlevel>ERROR) create_docu_array();
 
@@ -203,11 +216,24 @@ int main(int argc,char **argv)
   error(NOTE,string);
   
   time(&compilation_end);
+
+  if (to_bind) {
+    struct libfile_name *lib;
+    if (mybind(to_bind)) {
+      sprintf(string,"Successfully bound '%s' and '%s' into '%s'",interpreter_path,main_file_name,to_bind);
+      error(INFO,string);
+      end_it();
+    } else {
+      sprintf(string,"could not bind '%s' and '%s' into '%s'",interpreter_path,main_file_name,to_bind);
+      error(ERROR,string);
+      end_it();
+    }
+  }
   
   if (errorlevel>ERROR && !check_compat) {
     program_state=RUNNING;
-    run_it();}
-  else {
+    run_it();
+  } else {
     program_state=FINISHED;
     if (check_compat) 
       printf("Check for possible compatibility problems done\nProgram will not be executed, %d possible problem(s) reported\n",warning_count);
@@ -217,13 +243,13 @@ int main(int argc,char **argv)
   
   program_state=FINISHED;
   sprintf(string,"%d debug(s), %d note(s), %d warning(s), %d error(s)",
-    debug_count,note_count,warning_count,error_count);
+	  debug_count,note_count,warning_count,error_count);
   error(NOTE,string);
   time(&execution_end);
   sprintf(string,"compilation time %g second(s), execution %g",
-    (double)(compilation_end-compilation_start),(double)(execution_end-compilation_end));
+	  (double)(compilation_end-compilation_start),(double)(execution_end-compilation_end));
   error(NOTE,string);
-  end_it(endreason);
+  end_it();
   return !(errorlevel>ERROR);
 }
 
@@ -321,7 +347,7 @@ static void std_diag(char *head,int type,char *name) /* produce standard diagnos
 
 
 struct command *add_command(int type,char *name) 
-/* get room for new command, and make a link from old one */
+     /* get room for new command, and make a link from old one */
 {
   struct command *new;
   
@@ -378,7 +404,6 @@ static void parse_arguments(int cargc,char *cargv[])
   FILE *inputfile=NULL;
   char *option;
   char *info;
-  char *main_file_name=NULL;
   int options_done=FALSE;
   
   if (cargc>1) larg=strlen(cargv[1]);
@@ -414,13 +439,18 @@ static void parse_arguments(int cargc,char *cargv[])
   yabargv=(char **)my_malloc((larg+cargc+1)*sizeof(char *));
   yabargc=0;
   
+  if (is_bound) options_done=1;
   /* process options */
   for(ar=1;ar<argc;ar++) {
     option=argv[ar];
     if (!options_done) {
-      if (!strncmp("-help",option,2) || !strncmp("-?",option,2)) {
+      if (equal("-help",option,-2) || 
+	  equal("-?",option,2) || 
+	  equal("-version",option,2) ||
+	  equal("-licence",option,4) || 
+	  equal("-license",option,4)) {
 	do_help(option);
-	end_it(endreason);
+	end_it();
       } else if (equal("-check",option,2)) {
 	check_compat=1;
       } else if (!strcmp("--",option)) {
@@ -430,7 +460,7 @@ static void parse_arguments(int cargc,char *cargv[])
 	ar++;
 	if (ar>=argc) {
 	  error(ERROR,"no infolevel specified " YABFORHELP);
-	  end_it(endreason);
+	  end_it();
 	}
 	info=argv[ar];
 	if (!strncmp(info,"debug",strlen(info))) infolevel=DEBUG;
@@ -442,14 +472,14 @@ static void parse_arguments(int cargc,char *cargv[])
 	else {
 	  sprintf(string,"there's no infolevel '%s' " YABFORHELP,argv[ar]);
 	  error(ERROR,string);
-	  end_it(endreason);
+	  end_it();
 	}
       }
       else if (equal("-fg",option,3) || equal("-foreground",option,4)) {   
 	ar++;
 	if (ar>=argc) {
 	  error(ERROR,"no foreground colour specified " YABFORHELP);
-	  end_it(endreason);
+	  end_it();
 	}
 	foreground=my_strdup(argv[ar]);
       }
@@ -457,7 +487,7 @@ static void parse_arguments(int cargc,char *cargv[])
 	ar++;
 	if (ar>=argc) {
 	  error(ERROR,"no background colour specified (-h for help)");
-	  end_it(endreason);
+	  end_it();
 	}
 	background=my_strdup(argv[ar]);   
       }
@@ -465,23 +495,31 @@ static void parse_arguments(int cargc,char *cargv[])
 	ar++;
 	if (ar>=argc) {
 	  error(ERROR,"no geometry string specified (-h for help)");
-	  end_it(endreason);
+	  end_it();
 	}
 	geometry=my_strdup(argv[ar]);     
       }
+      else if (equal("-bind",option,3)) {
+	ar++;
+	if (ar>=argc) {
+	  error(ERROR,"name of bound program to be written is missing");
+	  end_it();
+	}
+	to_bind=my_strdup(argv[ar]);
+      }
       else if (equal("-execute",option,2)) {             
 	ar++;
 	if (ar>=argc) {
-	  error(ERROR,"no command specified (-h for help)");
-	  end_it(endreason);
+	  error(ERROR,"no commands specified (-h for help)");
+	  end_it();
 	}
 	explicit=my_strdup(argv[ar]);     
       }
-      else if (equal("-librarypath",option,3)) {             
+      else if (equal("-librarypath",option,4)) {             
 	ar++;
 	if (ar>=argc) {
 	  error(ERROR,"no library path specified (-h for help)");
-	  end_it(endreason);
+	  end_it();
 	}
 	strcpy(library_path,argv[ar]);
       }
@@ -489,7 +527,7 @@ static void parse_arguments(int cargc,char *cargv[])
 	ar++;
 	if (ar>=argc) {
 	  error(ERROR,"no display name specified (-h for help)");
-	  end_it(endreason);
+	  end_it();
 	}
 	displayname=my_strdup(argv[ar]);
       }
@@ -497,7 +535,7 @@ static void parse_arguments(int cargc,char *cargv[])
 	ar++;
 	if (ar>=argc) {
 	  error(ERROR,"no font specified (-h for help)");
-	  end_it(endreason);
+	  end_it();
 	}
 	font=my_strdup(argv[ar]);
       } else if (!print_docu &&
@@ -507,32 +545,32 @@ static void parse_arguments(int cargc,char *cargv[])
 	if (!strncmp("-doc_",option,5)) {
 	  ar--;
 	  hold_docu=TRUE;
-	  main_file_name=option+5;
+	  main_file_name=my_strdup(option+5);
 	} else if (!strncmp("-docu_",option,6)) {
 	  ar--;
 	  hold_docu=TRUE;
-	  main_file_name=option+6;
+	  main_file_name=my_strdup(option+6);
 	} else {
 	  if (ar>=argc-1) {
 	    error(ERROR,"no filename specified (-h for help)");
-	    end_it(endreason);
+	    end_it();
 	  }
 	  hold_docu=FALSE;
-	  main_file_name=argv[ar+1];
+	  main_file_name=my_strdup(argv[ar+1]);
 	}
       } else if (!print_docu && *option=='-') {
 	sprintf(string,"unknown or ambigous option '%s' " YABFORHELP,option);
 	error(ERROR,string);
-	end_it(endreason);
-      } else if (!inputfile && !explicit) { /* not an option */
-	if (!main_file_name) main_file_name=argv[ar];
+	end_it();
+      } else if (!is_bound && !inputfile && !explicit) { /* not an option */
+	if (!main_file_name) main_file_name=my_strdup(argv[ar]);
 	inputfile=fopen(main_file_name,"r");
 	if (inputfile==NULL) {
-	  sprintf(string,"could not open '%s'",main_file_name);
+	  sprintf(string,"could not open '%s': %s",main_file_name,my_strerror(errno));
 	  error(ERROR,string);
 	  endreason=erERROR;
 	  exitcode=1;
-	  end_it(endreason);
+	  end_it();
 	} else {
 	  progname=strrchr(main_file_name,'\\');
 	  if (!progname) progname=strrchr(main_file_name,'/');
@@ -541,9 +579,6 @@ static void parse_arguments(int cargc,char *cargv[])
 	  else 
 	    progname=main_file_name;
 	  if (!progname) progname="Yabasic";
-#ifdef WINDOWS
-	  SetConsoleTitle(progname);
-#endif
 	}
       } else {    /* option for yabasic program */
 	yabargv[yabargc]=my_strdup(argv[ar]);
@@ -556,8 +591,19 @@ static void parse_arguments(int cargc,char *cargv[])
   }
   
   interactive=FALSE;
-
-  if (!inputfile && !explicit) {
+  
+#ifdef WINDOWS
+  if (is_bound || !progname) {
+    SetConsoleTitle("");
+  } else {
+    SetConsoleTitle(progname);
+  }
+#endif
+  
+  if (is_bound) {
+    inputfile=bound_program;
+    main_file_name=my_strdup(interpreter_path);
+  } else if (!inputfile && !explicit) {
     interactive=TRUE;
     inputfile=stdin;
     main_file_name="standard input";
@@ -566,34 +612,48 @@ static void parse_arguments(int cargc,char *cargv[])
 
   /* open a flex buffer for the initial file */
   open_main(inputfile,explicit,main_file_name);
- return;
+  return;
 
 }
 
 void do_help(char *op) /* process help option */ 
 {
+  char *ooop=op;
   char *oop=op;
+  if (op[0]=='-' && op[1]=='-') op++;
+  oop=op;
   op=strchr(++op,'-');
   if (!op) {
-    fprintf(stderr,"%s",BANNER);
-    fprintf(stderr,"For more help try one of these options:\n");
-    fprintf(stderr,"  -help-license : show license and conditions of copying\n");
-    fprintf(stderr,"  -help-usage   : invoking yabasic and commandline options\n\n");
-    fprintf(stderr,"See the file yabasic.htm (which comes with yabasic) for a description\n");
-    fprintf(stderr,"of the language, or go to www.yabasic.de for the latest information\n");
-    fprintf(stderr,"and other yabasic resources.\n\n");
+    if (equal("-version",oop,2)) {
+      fprintf(stderr,"%s\n",BANNER_VERSION);
+    } else if (equal("-license",oop,2) || equal("-licence",oop,2)) {
+      fprintf(stderr,"\n%s\n",YABLICENSE);
+    } else {
+      fprintf(stderr,"%s",BANNER);
+      fprintf(stderr,"For more help try one of these options:\n");
+      fprintf(stderr,"  -help-license : show license and conditions of copying\n");
+      fprintf(stderr,"  -help-usage   : invoking yabasic and commandline options\n\n");
+#ifdef UNIX
+      fprintf(stderr,"Type 'man yabasic' or see the file yabasic.htm for a description\n");
+#else
+      fprintf(stderr,"See the file yabasic.htm (accessible from the start-menu) for a\ndescription");
+#endif    
+      fprintf(stderr,"of the language, or go to www.yabasic.de for the latest\n");
+      fprintf(stderr,"information and other yabasic resources.\n\n");
+    }
   } else if (equal("-license",op,2) || equal("-licence",op,2)) {
     fprintf(stderr,"\n%s\n",YABLICENSE);
   } else if (equal("-usage",op,2)) {
-    fprintf(stderr,"\nusage:        yabasic [options] [filename [arguments]]\n\n");
-    fprintf(stderr,"options   : see below for a list of valid options\n");
-    fprintf(stderr,"filename  : file, which contains the yabasic program; omit it to type\n");
+    fprintf(stderr,"\nUsage:        yabasic [OPTIONS] [FILENAME [ARGUMENTS]]\n\n");
+    fprintf(stderr,"FILENAME  : file, which contains the yabasic program; omit it to type\n");
     fprintf(stderr,"            in your program on the fly (terminated by a double newline)\n");
-    fprintf(stderr,"arguments : these values are available from within the yabasic program\n\n");
-    fprintf(stderr,"available options:\n");
+    fprintf(stderr,"ARGUMENTS : strings, that are available from within the yabasic program\n\n");
+    fprintf(stderr,"available OPTIONS:\n");
     fprintf(stderr,"   -help               : print help message and other help options\n");
+    fprintf(stderr,"   -version            : show version of yabasic\n");
     fprintf(stderr,"   -infolevel [dnwefb] : set infolevel to debug,note,warning,error or fatal\n");
-    fprintf(stderr,"   -execute COMMANDS   : execute yabasic commands right away\n");
+    fprintf(stderr,"   -execute COMMANDS   : execute yabasic COMMANDS right away\n");
+    fprintf(stderr,"   -bind BOUND         : bind interpreter with FILENAME into BOUND\n");
     fprintf(stderr,"   -geometry x+y       : position graphic window at x,y\n");
 #ifdef UNIX
     fprintf(stderr,"   -fg,-bg COL         : specify fore/background color of graphic window\n");
@@ -605,12 +665,12 @@ void do_help(char *op) /* process help option */
 #endif
     fprintf(stderr,"   -docu NAME          : print embedded docu of program or library\n");
     fprintf(stderr,"   -check              : check for possible compatibility problems\n");
-    fprintf(stderr,"   --                  : pass any subsequent options as arguments to yabasic\n");
+    fprintf(stderr,"   --                  : pass any subsequent words as arguments to yabasic\n");
     fprintf(stderr,"   -librarypath PATH   : directory to search libraries not found in\n");
     fprintf(stderr,"                         current dir (default %s)\n",library_default);
     fprintf(stderr,"\n");
   } else {
-    sprintf(string,"unknown or ambigous option '%s' " YABFORHELP,oop);
+    sprintf(string,"unknown or ambigous option '%s' " YABFORHELP,ooop);
     error(ERROR,string);
   }
 }
@@ -619,8 +679,13 @@ void do_help(char *op) /* process help option */
 int equal(char *a,char *b,int min) /* helper for processing options */
 {
   int len;
-  if (b[0]=='-' && b[1]=='-') b++;
-  len=strlen(b);
+  if (b[0]=='-' && b[1]=='-' && b[2]) b++;
+  if (min<0) {
+    min=-min;
+    len=min;
+  } else {
+    len=strlen(b);
+  }
 
   return (!strncmp(a,b,len) && len>=min);
 }
@@ -628,7 +693,7 @@ int equal(char *a,char *b,int min) /* helper for processing options */
 
 #ifdef WINDOWS
 static void chop_command(char *command,int *argc,char ***argv)
-/* chop the WIN95-commandline into seperate strings */
+     /* chop the WIN95-commandline into seperate strings */
 {
   int i,j,count;
   int quote;
@@ -680,7 +745,7 @@ static void chop_command(char *command,int *argc,char ***argv)
 #endif
 
 
-static void end_it(int er) /* perform shutdown-operations */
+static void end_it(void) /* perform shutdown-operations */
 {
   char l[2];
 #ifdef UNIX
@@ -692,9 +757,9 @@ static void end_it(int er) /* perform shutdown-operations */
     waitpid(backpid,&status,0);
     backpid=-1;
   }
-  if ((curinized || winopened) && er!=erREQUEST) {
+  if ((curinized || winopened) && endreason!=erREQUEST) {
 #else
-    if (!Commandline && er!=erREQUEST) {
+    if (!Commandline && endreason!=erREQUEST) {
 #endif
       myswitch(STDIO_STREAM);
       onestring("---Program done, press RETURN---\n");
@@ -724,7 +789,7 @@ static void end_it(int er) /* perform shutdown-operations */
 
 
 static void initialize(void) 
-/* give correct values to pointers etc ... */
+     /* give correct values to pointers etc ... */
 {
   struct symbol *s;
   struct stackentry *base;
@@ -931,6 +996,7 @@ static void initialize(void)
   explanation[cDOCU]="DOCU";
   explanation[cEND]="END";
   explanation[cEXIT]="EXIT";
+  explanation[cBIND]="BIND";
   explanation[cERROR]="ERROR";
   explanation[cSTARTFOR]="STARTFOR";
   explanation[cFORCHECK]="FORCHECK";
@@ -1038,7 +1104,7 @@ void signal_handler(int sig)   /* handle signals */
 
 
 static void run_it()
-/* execute the compiled code */
+     /* execute the compiled code */
 {
   int l=0;
 
@@ -1252,6 +1318,8 @@ static void run_it()
 	forincrement(); DONE;
       case cSTARTFOR:
 	startfor(); DONE;
+      case cBIND:
+	mybind(pop(stSTRING)->pointer); DONE;
       case cEND:
 	endreason=erEOF; break;
       case cEXIT:
@@ -1279,7 +1347,7 @@ static void run_it()
 
 
 void error(int severity, char *messageline) 
-/* reports an basic error to the user and possibly exits */
+     /* reports an basic error to the user and possibly exits */
 {
   if (program_state==COMPILING)
     error_with_line(severity,messageline,yylineno);
@@ -1291,7 +1359,7 @@ void error(int severity, char *messageline)
 
 
 void error_with_line(int severity, char *message,int line) 
-/* reports an basic error to the user and possibly exits */
+     /* reports an basic error to the user and possibly exits */
 {
   char *callstack;
   char *stext;
@@ -1306,6 +1374,9 @@ void error_with_line(int severity, char *message,int line)
 #endif
     
     switch(severity) {
+    case(INFO): 
+      stext="---Info"; 
+      break;
     case(DUMP): 
       stext="---Dump"; 
       break;
@@ -1358,7 +1429,7 @@ void error_with_line(int severity, char *message,int line)
   if (severity<=FATAL) {
     program_state=FINISHED;
     fprintf(stderr,"---Immediate exit to system, due to a fatal error.\n");
-    end_it(endreason);
+    end_it();
   }
 #ifdef UNIX
   if (curinized && severity<=infolevel) reset_prog_mode();
@@ -1417,7 +1488,12 @@ struct libfile_name *new_file(char *l,char *s) /* create a new structure for lib
   
   /* check, if library has already been included */
   for(curr=libfile_stack[0];curr;curr=curr->next) {
-    if (!strcmp(curr->l,l)) return NULL;
+    if (!strcmp(curr->l,l)) {
+      if (is_bound) 
+	return curr;
+      else
+	return NULL;
+    }
   }
 
   new=my_malloc(sizeof(struct libfile_name));
@@ -1601,3 +1677,224 @@ void create_docu_array(void) /* create array with documentation */
   }
   get_sym("main.docu$",syARRAY,amADD_GLOBAL)->pointer=ar;
 } 
+
+
+int isbound(void) /* check if this interpreter is bound to a program */
+{
+  FILE *interpreter;
+  int i;
+  int c;
+  int proglen=0;
+  int bound=1;
+  
+  if (!interpreter_path || !interpreter_path[0]) {
+    error(FATAL,"interpreter_path is not set !");
+    return 0;
+  }
+  if (!(interpreter=fopen(interpreter_path,"r"))) {
+    sprintf(string,"Couldn't open '%s' to check, if it is bound: %s",interpreter_path,my_strerror(errno));
+    error(WARNING,string);
+    return 0;
+  }
+
+  if (fseek(interpreter,0-strlen(YABMAGIC)-1,SEEK_END)) {
+    sprintf(string,"Couldn't seek within '%s': %s",interpreter_path,my_strerror(errno));
+    error(WARNING,string);
+    return 0;
+  }
+  for(i=0;i<strlen(YABMAGIC);i++) {
+    c=fgetc(interpreter);
+    if (c==EOF || c!=(YABMAGIC)[i]) bound=0;
+  }
+  if (!bound) {
+    fclose(interpreter);
+    return bound;
+  }
+  
+  if (fseek(interpreter,0-strlen(YABMAGIC)-5-8-1,SEEK_END)) {
+    sprintf(string,"Couldn't seek within '%s': %s",interpreter_path,my_strerror(errno));
+    error(WARNING,string);
+    return 0;
+  }
+  if (!fscanf(interpreter,"%d",&proglen)) {
+    error(WARNING,"Could not read length of embedded program");
+    return 0;
+  }
+
+  if (fseek(interpreter,0-strlen(YABMAGIC)-5-8-5-5-proglen,SEEK_END)) {
+    sprintf(string,"Couldn't seek within '%s': %s",interpreter_path,my_strerror(errno));
+    error(WARNING,string);
+    return 0;
+  }
+
+  if (infolevel>=NOTE) {
+    error(NOTE,"Dumping the embedded program, that will be executed:");
+    fprintf(stderr,"     ");
+    for(i=0;i<proglen;i++) {
+      c=fgetc(interpreter);
+      fprintf(stderr,"%c",c);
+      if (c=='\n' && i<proglen-1) fprintf(stderr,"     ");
+    }
+    error(NOTE,"End of program, that will be executed");
+    if (fseek(interpreter,0-strlen(YABMAGIC)-5-8-5-5-proglen,SEEK_END)) {
+      sprintf(string,"Couldn't seek within '%s': %s",interpreter_path,my_strerror(errno));
+      error(WARNING,string);
+      return 0;
+    }
+  }
+  bound_program=interpreter;
+  return 1;
+}
+
+
+static int mybind(char *bound) /* bind a program to the interpreter and save it */
+{
+  FILE *fyab;
+  FILE *fprog;
+  FILE *fbound;
+  FILE *flib;
+  int c;
+  char *pc;
+  int i;
+  int proglen=0;
+
+  if (interactive) {
+    error(ERROR,"cannot bind a program when called interactive");
+    return 0;
+  }
+
+  if (!strcmp(interpreter_path,bound)) {
+    sprintf(string,"will not overwrite '%s' with '%s'",bound,interpreter_path);
+    error(ERROR,string);
+    return 0;
+  }
+  if (!strcmp(main_file_name,bound)) {
+    sprintf(string,"will not overwrite '%s' with '%s'",bound,main_file_name);
+    error(ERROR,string);
+    return 0;
+  }
+
+  if (!(fyab=fopen(interpreter_path,"rb"))) {
+    sprintf(string,"could not open '%s' for reading: %s",interpreter_path,my_strerror(errno));
+    error(ERROR,string);
+    return 0;
+  }
+  if (!(fprog=fopen(main_file_name,"rb"))) {
+    sprintf(string,"could not open '%s' for reading: %s",main_file_name,my_strerror(errno));
+    error(ERROR,string);
+    fclose(fyab);
+    return 0;
+  }
+  if (!(fbound=fopen(bound,"wb"))) {
+    sprintf(string,"could not open '%s' for writing: %s",bound,my_strerror(errno));
+    error(ERROR,string);
+    fclose(fyab);
+    fclose(fprog);
+    return 0;
+  }
+
+  if (infolevel>=DEBUG) {
+    sprintf(string,"binding %s and %s into %s",interpreter_path,main_file_name,bound);
+    error(NOTE,string);
+  }
+  
+  while((c=fgetc(fyab))!=EOF) {
+    fputc(c,fbound);
+  }
+  for (i=1;i<libfile_chain_length;i++) {
+     if (!(flib=fopen(libfile_chain[i]->l,"rb"))) {
+      sprintf(string,"could not open '%s' for reading: %s",libfile_chain[i]->l,my_strerror(errno));
+      error(ERROR,string);
+      fclose(flib);
+      return 0;
+    }
+    sprintf(string,"\nimport %s\n",libfile_chain[i]->s);
+    for (pc=string;*pc;pc++) {
+      fputc(*pc,fbound);
+      proglen++;
+    }
+    while((c=fgetc(flib))!=EOF) {
+      fputc(c,fbound);
+      proglen++;
+    }
+  }
+
+  for (pc="\nimport main\n";*pc;pc++) {
+    fputc(*pc,fbound);
+    proglen++;
+  }
+  for (pc="\nimport __END_OF_IMPORT\n";*pc;pc++) {
+    fputc(*pc,fbound);
+    proglen++;
+  }    
+  while((c=fgetc(fprog))!=EOF) {
+    fputc(c,fbound);
+    proglen++;
+  }
+  fprintf(fbound,"\nend\n");
+  fprintf(fbound,"rem %08d\n",proglen);
+  fprintf(fbound,"rem %s\n",YABMAGIC);
+  fclose(fyab);
+  fclose(fprog);
+  fclose(fbound);
+  
+  return 1;
+}
+
+char *find_interpreter(char *name) /* find interpreter with full path */
+{
+  FILE *f;
+  char *path=NULL;
+
+#ifdef WINDOWS
+  path=my_malloc(1000);
+  GetModuleFileName(NULL,path,1000);
+  if (f=fopen(path,"r")) {
+    fclose(f);
+    return path;
+  } else {
+    my_free(path);
+    return my_strdup(name);
+  }
+  
+#else	
+  if (f=fopen(name,"r")) {
+    fclose(f);
+    path=my_strdup(name);
+    return path;
+  } else {
+    char *from,*to,*try;
+    from=to=path=getenv("PATH");
+    try=my_malloc(strlen(path)+strlen(name)+2);
+    to=strchr(to+1,':');
+    while(to) {
+      strncpy(try,from,to-from);
+      try[to-from]=0;
+      if (try[to-from-1]!='/') strcat(try,"/");
+      strcat(try,name);
+      if (f=fopen(try,"r")) {
+	fclose(f);
+	return try;
+      }
+      from=to+1;
+      to=strchr(to+1,':');
+    }
+    return name;
+  }
+#endif
+}
+
+	
+char *my_strerror(int err) { /* return description of error */
+#ifdef WINDOWS
+  return strerror(err);
+#else
+#ifdef HAVE_STRERROR
+  return strerror(err);
+#else
+  char buff[100];
+  sprintf(buff,"errno=%d",err);
+  return buff;
+#endif
+#endif
+}
