@@ -1,7 +1,7 @@
 /*  
 
     YABASIC ---  a simple Basic Interpreter
-    written by Marc-Oliver Ihm 1995-2002
+    written by Marc-Oliver Ihm 1995-2004
     homepage: www.yabasic.de
     
     yabasic.h --- function prototypes and global variables
@@ -15,20 +15,25 @@
 
 
 #define YABLICENSE \
-"  Yabasic may be copied only under the terms of either the               \n"\
+"  Yabasic may only be copied under the terms of either the               \n"\
 "  Artistic License or the GNU General Public License (GPL),              \n"\
-"  both of which can be found at www.yabasic.de                           \n"\
+"  both of which can be found at www.yabasic.de.                          \n"\
+"  The choice, which license you want to apply, is yours,                 \n"\
+"  but you must stick to the license you have chosen for                  \n"\
+"  at least one version of yabasic.                                       \n"\
 "\n"\
-"  Earlier versions of yabasic have been subject to the GPL alone and     \n"\
-"  you may still choose to use and distribute yabasic under this license. \n"\
+"  Most people will want to use and distribute yabasic under the          \n"\
+"  terms of the Artistic License. It gives you the right to use           \n"\
+"  and distribute yabasic in a more-or-less customary fashion,            \n"\
+"  plus the right to make reasonable modifications and distribute         \n"\
+"  (or even sell) such a modified version under a different name.         \n"\
+"  However, the original author and copyright holder still reserves       \n"\
+"  himself some sort of artistic control over the development             \n"\
+"  of yabasic itself.                                                     \n"\
 "\n"\
-"  Recent versions may be used and distributed under the terms of         \n"\
-"  the Artistic License as well. It gives you the right to use and        \n"\
-"  distribute yabasic in a more-or-less customary fashion,                \n"\
-"  plus the right to make reasonable modifications and even sell          \n"\
-"  such a modified version. However, the original author and              \n"\
-"  copyright holder still reserves himself some sort of artistic          \n"\
-"  control over the development of yabasic itself.                        \n"
+"  However, as many people already know the GPL, yabasic may be           \n"\
+"  distributed under this license as well.                                \n"
+
 
 
 #define YABASIC_INCLUDED
@@ -56,18 +61,20 @@ UNIX and WINDOWS are defined at once; check your compiler settings
 #include <math.h>
 #include <time.h>
 #include <limits.h>
-
+#include <errno.h>
 #ifdef WINDOWS
 #include <string.h>
 #include <windows.h>
 #include <io.h>
+#define ARCHITECTURE "windows"
 #ifdef __LCC__  /* fix for lccwin32 */
 #include <winspool.h>
 #endif
 #endif
 
-#ifdef UNIX
 
+#ifdef UNIX
+#define ARCHITECTURE UNIX_ARCHITECTURE
 #ifdef HAS_STRING_HEADER
 #include <string.h>
 #elif HAS_STRINGS_HEADER
@@ -80,8 +87,6 @@ UNIX and WINDOWS are defined at once; check your compiler settings
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
 #include <X11/Intrinsic.h>
-#include <errno.h>
-extern int errno;
 #define XK_LATIN1
 #define XK_MISCELLANY
 #include <X11/keysymdef.h>
@@ -138,6 +143,7 @@ extern int errorcode; /* error-codes */
 extern char library_path[]; /* full path to search libraries */
 extern int program_state;  /* state of program */
 extern int check_compat; /* true, if compatibility should be checked */
+extern int is_bound; /* true, if this executable is bound */
 
 
 /* io.c */
@@ -225,6 +231,9 @@ extern struct libfile_name *libfile_stack[]; /* stack for library file names */
 extern struct libfile_name *currlib; /* current libfile as relevant to bison */
 extern int inlib; /* true, while in library */
 extern int fi_pending; /* true, if within a short if */
+extern int libfile_chain_length; /* length of libfile_chain */
+extern struct libfile_name *libfile_chain[]; /* list of all library file names */
+
 
 /* bison.c */
 extern char *current_function; /* name of currently parsed function */
@@ -264,7 +273,7 @@ extern int in_loop;
 /* ---------------------- enum types ------------------------------- */
 
 enum error {  /* error levels  */
-  FATAL,ERROR,DUMP,WARNING,NOTE,DEBUG
+  FATAL,ERROR,INFO,DUMP,WARNING,NOTE,DEBUG
 };
 
 enum endreasons { /* ways to end the program */
@@ -282,10 +291,10 @@ enum functions { /* functions in yabasic (sorted by number of arguments) */
   fSIN,fASIN,fCOS,fACOS,fTAN,
   fATAN,fSYSTEM,fSYSTEM2,fPEEK,fPEEK2,fPEEK4,fTELL,fEXP,fLOG,fLEN,fSTR,
   fSQRT,fSQR,fFRAC,fABS,fSIG,fRAN,fINT,fVAL,fASC,fHEX,fBIN,fDEC,fUPPER,fLOWER,
-  fLTRIM,fRTRIM,fTRIM,fCHR,fONEARGS,fDEC2,fATAN2,fLEFT,fAND,fOR,fEOR,
+  fLTRIM,fRTRIM,fTRIM,fCHR,fONEARGS,fDEC2,fATAN2,fLEFT,fAND,fOR,fEOR,fLOG2,
   fRIGHT,fINSTR,fRINSTR,fSTR2,fMOD,fMIN,fMAX,fPEEK3,fMID2,
   fTWOARGS,
-  fMID,fINSTR2,fRINSTR2,
+  fMID,fINSTR2,fRINSTR2,fSTR3,
   fTHREEARGS,
   fGETBIT,fGETCHAR
 };
@@ -302,8 +311,8 @@ enum cmd_type { /* type of command */
   cFIRST_COMMAND, /* no command, just marks start of list */
   
   cLABEL,cSUBLINK,cGOTO,cQGOTO,cGOSUB,cQGOSUB,cRETURN,  /* flow control */
-  cEND,cEXIT,cDECIDE,cSKIPPER,cNOP,cFINDNOP,cEXCEPTION,cANDSHORT,cORSHORT,
-  cSKIPONCE,cRESETSKIPONCE,cCOMPILE,cEXECUTE,cEXECUTE2,
+  cEND,cEXIT,cBIND,cDECIDE,cSKIPPER,cNOP,cFINDNOP,cEXCEPTION,cANDSHORT,
+  cORSHORT,cSKIPONCE,cRESETSKIPONCE,cCOMPILE,cEXECUTE,cEXECUTE2,
   
   cDIM,cFUNCTION,cDOARRAY,cARRAYLINK,cPUSHARRAYREF,cCLEARREFS,   /* everything with "()" */
   cARDIM,cARSIZE,cTOKEN,cTOKEN2,cTOKENALT,cTOKENALT2,
@@ -460,6 +469,7 @@ void error(int,char *); /* reports an error and possibly exits */
 void error_with_line(int,char *,int); /* reports an error and possibly exits */
 void *my_malloc(unsigned); /* my own version of malloc */
 void my_free(void *); /* free memory */
+char *my_strerror(int); /* return description of error messages */
 struct command *add_command(int,char *); /* get room for new command */
 void signal_handler(int);  /* handle various signals */
 char *my_strdup(char *); /* my own version of strdup */
@@ -473,6 +483,7 @@ extern void add_variables(char *); /* add pi and e to symbol table */
 void compile(void); /* create a subroutine at runtime */
 void create_execute(int); /* create command 'cEXECUTE' */
 void execute(struct command *); /* execute a subroutine */
+int isbound(void); /* check if this interpreter is bound to a program */
 
 
 /* io.c */
@@ -548,7 +559,7 @@ void create_boole(char); /* create command boole */
 void boole(struct command *);  /* perform and/or/not */
 void create_function(int); /* create command 'function' */
 void function(struct command *); /* performs a function */
-int myformat(char *,double,char *); /* format number */
+int myformat(char *,double,char *,char *); /* format number */
 void create_restore(char *); /* create command 'restore' */
 void restore(struct command *); /* reset data pointer to given label */
 void create_dbldata(double);  /* create command dbldata */
